package com.manage.dao;

import com.manage.entity.Power_User;
import com.manage.vo.Power_UserVo;
import com.manage.vo.User;

import java.util.List;

/**
 * @Author:hjl
 * @Date:Creatid in 1:09 2019/4/17
 * @Description:
 */
/*@Repository*/
public class Power_UserMapperImpl implements Power_UserMapper{
    @Override
    public int deleteByPrimaryKey(Integer userId) {
        return this.deleteByPrimaryKey(userId);
    }

    @Override
    public int insert(Power_User record) {
        return this.insert(record);
    }

    @Override
    public int insertSelective(Power_User record) {
        return this.insertSelective(record);
    }

    @Override
    public Power_UserVo selectByPrimaryKey(Integer userId) {
        return this.selectByPrimaryKey(userId);
    }

    @Override
    public int updateByPrimaryKeySelective(Power_User record) {
        return this.updateByPrimaryKey(record);
    }

    @Override
    public int updateByPrimaryKey(Power_User record) {
        return this.updateByPrimaryKey(record);
    }

    @Override
    public List<Power_User> checkUserName(String userName) {
        return this.checkUserName(userName);
    }

    @Override
    public Power_UserVo findPowerUserByUserNameAndUserPwd(Power_User powerUser) {
        return this.findPowerUserByUserNameAndUserPwd(powerUser);
    }

    @Override
    public List<Power_UserVo> findSomeByMore(Power_UserVo powerUser) {
        return this.findSomeByMore(powerUser);
    }

    @Override
    public int getTotal(Power_UserVo powerUser) {
        return this.getTotal(powerUser);
    }

    @Override
    public List<Power_UserVo> selectList() {
        return this.selectList();
    }

    @Override
    public List<Power_UserVo> selectAllByDeptId(Integer deptId,String userName) {
        return this.selectAllByDeptId(deptId,userName);
    }

    @Override
    public List<Power_UserVo> selectAllBySysRole(Integer dictId) {
        return this.selectAllBySysRole(dictId);
    }

    @Override
    public List<Power_UserVo> selectAllByRoleIdAndDictId(Integer roleId,Integer dictId) {
        return this.selectAllByRoleIdAndDictId(roleId,dictId);
    }

    @Override
    public Integer updatePassword(String userPwd, Integer userId, String updater, String updateDate) {
        return this.updatePassword(userPwd,userId, updater, updateDate);
    }

    @Override
    public Integer validPassword(String userPwd, Integer userId, String updater, String updateDate) {
        return this.validPassword(userPwd,userId, updater, updateDate);
    }

    @Override
    public List<Power_UserVo> selectAllBySysId(Integer hospitalId,String userName) {
        return this.selectAllBySysId(hospitalId,userName);
    }

    @Override
    public List<Power_UserVo> selectUserListByNoticeTypeId(Integer noticeTypeId,Integer hospitalId) {
        return this.selectUserListByNoticeTypeId(noticeTypeId,hospitalId);
    }

    @Override
    public Integer selectHospitalIdByUserId(Integer userId) {
        return this.selectHospitalIdByUserId(userId);
    }

    @Override
    public List<Power_UserVo> selectUsersBySysFlagAndHospitalId(String sysFlag, Integer hospitalId) {
        return this.selectUsersBySysFlagAndHospitalId(sysFlag,hospitalId);
    }

    @Override
    public List<User> selectUserIdAndUserNameList(Integer userId) {
        return this.selectUserIdAndUserNameList(userId);
    }

    @Override
    public Integer selectEffectiveCount() {
        return this.selectEffectiveCount();
    }

    @Override
    public List<Power_UserVo> selectAllNotAdmin() {
        return this.selectAllNotAdmin();
    }

    @Override
    public int SimpleInsert(List<Power_User> list) {
        return this.SimpleInsert(list);
    }

    @Override
    public Power_UserVo findPowerUserByUserNameAndRemark(Power_User powerUser) {
        return this.findPowerUserByUserNameAndRemark(powerUser);
    }

    @Override
    public List<Power_User> getUserTreeByHospitalId(Integer hospitalId) {
        return this.getUserTreeByHospitalId(hospitalId);
    }

    @Override
    public List<Power_User> getUserTreeByDeptId(Integer deptId) {
        return this.getUserTreeByDeptId(deptId);
    }
}
